@@ -68,18 +68,26 @@ class API(object):
             try:
                 images = self.rpcapi.image_search(
                     context, new_container.image,
-                    new_container.image_driver, True, host_state['host'])
+                    new_container.image_driver, True, new_container.registry,
+                    host_state['host'])
                 if not images:
                     raise exception.ImageNotFound(image=new_container.image)
                 if len(images) > 1:
                     raise exception.Conflict('Multiple images exist with same '
                                              'name. Please use the container '
                                              'uuid instead.')
+            except exception.OperationNotSupported:
+                LOG.info("Skip validation since search is not supported for "
+                         "image '%(image)s' and image driver '%(driver)s'.",
+                         {'image': new_container.image,
+                          'driver': new_container.image_driver})
+            except exception.ReferenceInvalidFormat:
+                raise exception.InvalidValue(_("The format of image name '%s' "
+                                               "is invalid.")
+                                             % new_container.image)
             except Exception as e:
-                new_container.status = consts.ERROR
-                new_container.status_reason = str(e)
-                new_container.save(context)
-                raise
+                LOG.warning("Skip validation since image search failed with "
+                            "unexpected exception: %s", str(e))
 
         self._record_action_start(context, new_container,
                                   container_actions.CREATE)
@@ -101,10 +109,10 @@ class API(object):
     def container_show(self, context, container):
         return self.rpcapi.container_show(context, container)
 
-    def container_rebuild(self, context, container):
+    def container_rebuild(self, context, container, run):
         self._record_action_start(context, container,
                                   container_actions.REBUILD)
-        return self.rpcapi.container_rebuild(context, container)
+        return self.rpcapi.container_rebuild(context, container, run)
 
     def container_reboot(self, context, container, *args):
         self._record_action_start(context, container, container_actions.REBOOT)
@@ -188,42 +196,16 @@ class API(object):
         self._record_action_start(context, container, container_actions.COMMIT)
         return self.rpcapi.container_commit(context, container, *args)
 
-    def image_delete(self, context, image, host):
-        return self.rpcapi.image_delete(context, image, host)
+    def image_delete(self, context, image):
+        return self.rpcapi.image_delete(context, image, image.host)
 
-    def image_pull(self, context, image, host):
-        return self.rpcapi.image_pull(context, image, host)
+    def image_pull(self, context, image):
+        return self.rpcapi.image_pull(context, image, image.host)
 
     def image_search(self, context, image, image_driver, exact_match, *args):
         return self.rpcapi.image_search(context, image, image_driver,
                                         exact_match, *args)
 
-    def capsule_create(self, context, new_capsule, requested_networks,
-                       requested_volumes, extra_spec):
-        try:
-            host_state = self._schedule_container(context, new_capsule,
-                                                  extra_spec)
-        except exception.NoValidHost:
-            new_capsule.status = consts.FAILED
-            new_capsule.status_reason = _(
-                "There are not enough hosts available.")
-            new_capsule.save(context)
-            return
-        except Exception:
-            new_capsule.status = consts.FAILED
-            new_capsule.status_reason = _("Unexpected exception occurred.")
-            new_capsule.save(context)
-            raise
-        for container in new_capsule.containers:
-            self._record_action_start(context, container,
-                                      container_actions.CREATE)
-        self.rpcapi.capsule_create(context, host_state['host'], new_capsule,
-                                   requested_networks, requested_volumes,
-                                   host_state['limits'])
-
-    def capsule_delete(self, context, capsule):
-        return self.rpcapi.capsule_delete(context, capsule)
-
     def network_detach(self, context, container, *args):
         self._record_action_start(context, container,
                                   container_actions.NETWORK_DETACH)
@@ -234,8 +216,11 @@ class API(object):
                                   container_actions.NETWORK_ATTACH)
         return self.rpcapi.network_attach(context, container, *args)
 
-    def network_create(self, context, network):
-        return self.rpcapi.network_create(context, network)
+    def network_create(self, context, *args):
+        return self.rpcapi.network_create(context, *args)
+
+    def network_delete(self, context, *args):
+        return self.rpcapi.network_delete(context, *args)
 
     def resize_container(self, context, container, *args):
         return self.rpcapi.resize_container(context, container, *args)
