@@ -20,10 +20,14 @@ from oslo_log import log as logging
 from zun.common import exception
 from zun.common import utils
 from zun.compute import claims
+import zun.conf
 from zun import objects
 from zun.objects import base as obj_base
+from zun.pci import manager as pci_manager
 from zun.scheduler import client as scheduler_client
 
+
+CONF = zun.conf.CONF
 LOG = logging.getLogger(__name__)
 COMPUTE_RESOURCE_SEMAPHORE = "compute_resources"
 
@@ -36,6 +40,17 @@ class ComputeNodeTracker(object):
         self.tracked_containers = {}
         self.old_resources = collections.defaultdict(objects.ComputeNode)
         self.scheduler_client = scheduler_client.SchedulerClient()
+        self.pci_tracker = None
+
+    def _setup_pci_tracker(self, context, compute_node):
+        if not self.pci_tracker:
+            n_id = compute_node.uuid
+            self.pci_tracker = pci_manager.PciDevTracker(context, node_id=n_id)
+            dev_json = self.container_driver.get_pci_resources()
+            self.pci_tracker.update_devices_from_compute_resources(dev_json)
+
+            dev_pools_obj = self.pci_tracker.stats.to_device_pools_obj()
+            compute_node.pci_device_pools = dev_pools_obj
 
     def update_available_resources(self, context):
         # Check if the compute_node is already registered
@@ -46,9 +61,13 @@ class ComputeNodeTracker(object):
             node = objects.ComputeNode(context)
             node.hostname = self.host
             node.numa_topology = numa_obj
+            node.disk_quota_supported = \
+                self.container_driver.node_support_disk_quota()
+            node.enable_cpu_pinning = CONF.compute.enable_cpu_pinning
             node.create(context)
             LOG.info('Node created for :%(host)s', {'host': self.host})
         self.container_driver.get_available_resources(node)
+        self._setup_pci_tracker(context, node)
         self.compute_node = node
         self._update_available_resource(context)
         # NOTE(sbiswas7): Consider removing the return statement if not needed
@@ -57,13 +76,13 @@ class ComputeNodeTracker(object):
     def _get_compute_node(self, context):
         """Returns compute node for the host"""
         try:
-            return objects.ComputeNode.get_by_hostname(context, self.host)
+            return objects.ComputeNode.get_by_name(context, self.host)
         except exception.ComputeNodeNotFound:
             LOG.warning("No compute node record for: %(host)s",
                         {'host': self.host})
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE)
-    def container_claim(self, context, container, hostname, limits=None):
+    def container_claim(self, context, container, pci_requests, limits=None):
         """Indicate resources are needed for an upcoming container build.
 
         This should be called before the compute node is about to perform
@@ -72,29 +91,67 @@ class ComputeNodeTracker(object):
         :param context: security context
         :param container: container to reserve resources for.
         :type container: zun.objects.container.Container object
-        :param hostname: The zun hostname selected by the scheduler
+        :param pci_requests: pci requests for sriov port.
         :param limits: Dict of oversubscription limits for memory, disk,
                        and CPUs.
         :returns: A Claim ticket representing the reserved resources.  It can
                   be used to revert the resource usage if an error occurs
                   during the container build.
         """
-        # No memory and cpu specified, no need to claim resource now.
-        if not (container.memory or container.cpu):
-            self._set_container_host(context, container)
+        # No memory, cpu, disk or pci_request specified, no need to claim
+        # resource now.
+        if not (container.memory or container.cpu or pci_requests or
+                container.disk):
             return claims.NopClaim()
 
         # We should have the compute node created here, just get it.
         self.compute_node = self._get_compute_node(context)
-        if self.disabled(hostname):
-            self._set_container_host(context, container)
-            return claims.NopClaim()
 
         claim = claims.Claim(context, container, self, self.compute_node,
-                             limits=limits)
+                             pci_requests, limits=limits)
+
+        if self.pci_tracker:
+            self.pci_tracker.claim_container(context, container.uuid,
+                                             pci_requests)
 
         self._set_container_host(context, container)
-        self._update_usage_from_container(container)
+        self._update_usage_from_container(context, container)
+        # persist changes to the compute node:
+        self._update(self.compute_node)
+
+        return claim
+
+    @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE)
+    def container_update_claim(self, context, new_container, old_container,
+                               limits=None):
+        """Indicate resources are needed for an upcoming container update.
+
+        This should be called before the compute node is about to perform
+        an container update operation that will consume additional resources.
+
+        :param context: security context
+        :param new_container: container to be updated to.
+        :type new_container: zun.objects.container.Container object
+        :param old_container: container to be updated from.
+        :type old_container: zun.objects.container.Container object
+        :param limits: Dict of oversubscription limits for memory, disk,
+                       and CPUs.
+        :returns: A Claim ticket representing the reserved resources.  It can
+                  be used to revert the resource usage if an error occurs
+                  during the container update.
+        """
+        if (new_container.cpu == old_container.cpu and
+                new_container.memory == old_container.memory):
+            return claims.NopClaim()
+
+        # We should have the compute node created here, just get it.
+        self.compute_node = self._get_compute_node(context)
+
+        claim = claims.UpdateClaim(context, new_container, old_container,
+                                   self, self.compute_node, limits=limits)
+
+        self._update_usage_from_container_update(context, new_container,
+                                                 old_container)
         # persist changes to the compute node:
         self._update(self.compute_node)
 
@@ -112,7 +169,8 @@ class ComputeNodeTracker(object):
         container.host = self.host
         container.save(context)
 
-    def _update_usage_from_container(self, container, is_removed=False):
+    def _update_usage_from_container(self, context, container,
+                                     is_removed=False):
         """Update usage for a single container."""
 
         uuid = container.uuid
@@ -129,15 +187,27 @@ class ComputeNodeTracker(object):
             sign = -1
 
         if is_new_container or is_removed_container:
-            # TODO(Shunli): Handle pci, scheduler allocation here.
+            if self.pci_tracker:
+                self.pci_tracker.update_pci_for_container(context, container,
+                                                          sign=sign)
 
             # new container, update compute node resource usage:
             self._update_usage(self._get_usage_dict(container), sign=sign)
 
+    def _update_usage_from_container_update(self, context, new_container,
+                                            old_container):
+        """Update usage for a container update."""
+        uuid = new_container.uuid
+        self.tracked_containers[uuid] = obj_base.obj_to_primitive(
+            new_container)
+        # update compute node resource usage
+        self._update_usage(self._get_usage_dict(old_container), sign=-1)
+        self._update_usage(self._get_usage_dict(new_container))
+
     def _update_usage_from_containers(self, context, containers):
         """Calculate resource usage based on container utilization.
 
-        This is different than the conatiner daemon view as it will account
+        This is different than the container daemon view as it will account
         for all containers assigned to the local compute host, even if they
         are not currently powered on.
         """
@@ -149,33 +219,54 @@ class ComputeNodeTracker(object):
         cn.mem_free = cn.mem_total
         cn.mem_used = 0
         cn.running_containers = 0
+        cn.disk_used = 0
 
         for cnt in containers:
-            self._update_usage_from_container(cnt)
+            self._update_usage_from_container(context, cnt)
 
         cn.mem_free = max(0, cn.mem_free)
 
     def _update_usage(self, usage, sign=1):
         mem_usage = usage['memory']
         cpus_usage = usage.get('cpu', 0)
+        disk_usage = usage['disk']
+        cpuset_cpus_usage = None
+        numa_node_id = 0
+        if 'cpuset_cpus' in usage.keys():
+            cpuset_cpus_usage = usage['cpuset_cpus']
+            numa_node_id = usage['node']
 
         cn = self.compute_node
+        numa_topology = cn.numa_topology.nodes
         cn.mem_used += sign * mem_usage
         cn.cpu_used += sign * cpus_usage
+        cn.disk_used += sign * disk_usage
 
         # free ram may be negative, depending on policy:
         cn.mem_free = cn.mem_total - cn.mem_used
 
         cn.running_containers += sign * 1
 
-        # TODO(Shunli): Calculate the numa usage here
+        if cpuset_cpus_usage:
+            for numa_node in numa_topology:
+                if numa_node.id == numa_node_id:
+                    numa_node.mem_available = (numa_node.mem_available -
+                                               mem_usage * sign)
+                    if sign > 0:
+                        numa_node.pin_cpus(cpuset_cpus_usage)
+                        cn._changed_fields.add('numa_topology')
+                    else:
+                        numa_node.unpin_cpus(cpuset_cpus_usage)
+                        cn._changed_fields.add('numa_topology')
 
     def _update(self, compute_node):
         if not self._resource_change(compute_node):
             return
         # Persist the stats to the Scheduler
         self.scheduler_client.update_resource(compute_node)
-        # Update pci tracker here
+
+        if self.pci_tracker:
+            self.pci_tracker.save()
 
     def _resource_change(self, compute_node):
         """Check to see if any resources have changed."""
@@ -226,21 +317,31 @@ class ComputeNodeTracker(object):
         # (Fixme): The Container.memory is string.
         memory = 0
         if container.memory:
-            memory = int(container.memory[:-1])
+            memory = int(container.memory)
         usage = {'memory': memory,
-                 'cpu': container.cpu or 0}
-
-        # update numa usage here
+                 'cpu': container.cpu or 0,
+                 'disk': container.disk or 0}
+        if container.cpuset.cpuset_cpus:
+            usage['cpuset_cpus'] = container.cpuset.cpuset_cpus
+            usage['node'] = int(container.cpuset.cpuset_mems)
 
         return usage
 
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE)
-    def abort_container_claim(self, container):
+    def abort_container_claim(self, context, container):
         """Remove usage from the given container."""
-        self._update_usage_from_container(container, is_removed=True)
+        self._update_usage_from_container(context, container, is_removed=True)
 
         self._update(self.compute_node)
 
+    @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE)
+    def abort_container_update_claim(self, context, new_container,
+                                     old_container):
+        """Remove usage from the given container."""
+        self._update_usage_from_container_update(context, old_container,
+                                                 new_container)
+        self._update(self.compute_node)
+
     @utils.synchronized(COMPUTE_RESOURCE_SEMAPHORE)
     def remove_usage_from_container(self, context, container,
                                     is_removed=True):
@@ -248,5 +349,5 @@ class ComputeNodeTracker(object):
 
         # We need to get the latest compute node info
         self.compute_node = self._get_compute_node(context)
-        self._update_usage_from_container(container, is_removed)
+        self._update_usage_from_container(context, container, is_removed)
         self._update(self.compute_node)
