@@ -12,14 +12,17 @@
 # limitations under the License.
 
 import datetime
+import errno
 import eventlet
 import functools
 import types
 
 from docker import errors
+from neutronclient.common import exceptions as n_exc
 from oslo_log import log as logging
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
+import psutil
 import six
 
 from zun.common import consts
@@ -27,7 +30,7 @@ from zun.common import exception
 from zun.common.i18n import _
 from zun.common import utils
 from zun.common.utils import check_container_id
-from zun.compute import api as zun_compute
+from zun.compute import container_actions
 import zun.conf
 from zun.container.docker import host
 from zun.container.docker import utils as docker_utils
@@ -103,25 +106,27 @@ def wrap_docker_error(function):
 
 class DockerDriver(driver.ContainerDriver):
     """Implementation of container drivers for Docker."""
-    capabilities = {
-        "support_sandbox": True,
-        "support_standalone": True,
-    }
 
     def __init__(self):
         super(DockerDriver, self).__init__()
         self._host = host.Host()
         self._get_host_storage_info()
-        self.volume_driver = vol_driver.driver()
         self.image_drivers = {}
         for driver_name in CONF.image_driver_list:
             driver = img_driver.load_image_driver(driver_name)
             self.image_drivers[driver_name] = driver
+        self.volume_drivers = {}
+        for driver_name in CONF.volume.driver_list:
+            driver = vol_driver.driver(driver_name)
+            self.volume_drivers[driver_name] = driver
 
     def _get_host_storage_info(self):
+        host_info = self.get_host_info()
+        self.docker_root_dir = host_info['docker_root_dir']
         storage_info = self._host.get_storage_info()
         self.base_device_size = storage_info['default_base_size']
-        self.support_disk_quota = self._host.check_supported_disk_quota()
+        self.support_disk_quota = self._host.check_supported_disk_quota(
+            host_info)
 
     def load_image(self, image_path=None):
         with docker_utils.docker_client() as docker:
@@ -161,7 +166,7 @@ class DockerDriver(driver.ContainerDriver):
         try:
             image_driver.delete_committed_image(context, img_id)
         except Exception as e:
-            LOG.exception('Unknown exception occurred while'
+            LOG.exception('Unknown exception occurred while '
                           'deleting image %s: %s',
                           img_id,
                           six.text_type(e))
@@ -172,16 +177,18 @@ class DockerDriver(driver.ContainerDriver):
             return docker.images(repo, quiet)
 
     def pull_image(self, context, repo, tag, image_pull_policy='always',
-                   driver_name=None):
+                   driver_name=None, registry=None):
         if driver_name is None:
             driver_name = CONF.default_image_driver
 
         try:
             image_driver = self.image_drivers[driver_name]
             image, image_loaded = image_driver.pull_image(
-                context, repo, tag, image_pull_policy)
+                context, repo, tag, image_pull_policy, registry)
             if image:
                 image['driver'] = driver_name.split('.')[0]
+        except exception.ZunException:
+            raise
         except Exception as e:
             LOG.exception('Unknown exception occurred while loading '
                           'image: %s', six.text_type(e))
@@ -197,6 +204,8 @@ class DockerDriver(driver.ContainerDriver):
             image_driver = self.image_drivers[driver_name]
             return image_driver.search_image(context, repo, tag,
                                              exact_match)
+        except exception.ZunException:
+            raise
         except Exception as e:
             LOG.exception('Unknown exception occurred while searching '
                           'for image: %s', six.text_type(e))
@@ -239,15 +248,14 @@ class DockerDriver(driver.ContainerDriver):
 
     def create(self, context, container, image, requested_networks,
                requested_volumes):
-        sandbox_id = container.get_sandbox_id()
-
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context=context, docker_api=docker)
             name = container.name
             LOG.debug('Creating container with image %(image)s name %(name)s',
                       {'image': image['image'], 'name': name})
             self._provision_network(context, network_api, requested_networks)
-            binds = self._get_binds(context, requested_volumes)
+            volmaps = requested_volumes.get(container.uuid, [])
+            binds = self._get_binds(context, volmaps)
             kwargs = {
                 'name': self.get_container_name(container),
                 'command': container.command,
@@ -256,11 +264,8 @@ class DockerDriver(driver.ContainerDriver):
                 'labels': container.labels,
                 'tty': container.interactive,
                 'stdin_open': container.interactive,
+                'hostname': container.hostname,
             }
-            if not sandbox_id:
-                # Sandbox is not used so it is legitimate to customize
-                # the container's hostname
-                kwargs['hostname'] = container.hostname
 
             if not self._is_runtime_supported():
                 if container.runtime:
@@ -275,16 +280,10 @@ class DockerDriver(driver.ContainerDriver):
             host_config['runtime'] = runtime
             host_config['binds'] = binds
             kwargs['volumes'] = [b['bind'] for b in binds.values()]
-            if sandbox_id:
-                host_config['network_mode'] = 'container:%s' % sandbox_id
-                # TODO(hongbin): Uncomment this after docker-py add support for
-                # container mode for pid namespace.
-                # host_config['pid_mode'] = 'container:%s' % sandbox_id
-                host_config['ipc_mode'] = 'container:%s' % sandbox_id
-            else:
-                self._process_networking_config(
-                    context, container, requested_networks, host_config,
-                    kwargs, docker)
+            self._process_exposed_ports(network_api.neutron_api, container)
+            self._process_networking_config(
+                context, container, requested_networks, host_config,
+                kwargs, docker)
             if container.auto_remove:
                 host_config['auto_remove'] = container.auto_remove
             if container.memory is not None:
@@ -301,6 +300,9 @@ class DockerDriver(driver.ContainerDriver):
             if container.disk:
                 disk_size = str(container.disk) + 'G'
                 host_config['storage_opt'] = {'size': disk_size}
+            if container.cpu_policy == 'dedicated':
+                host_config['cpuset_cpus'] = container.cpuset.cpuset_cpus
+                host_config['cpuset_mems'] = str(container.cpuset.cpuset_mems)
             # The time unit in docker of heath checking is us, and the unit
             # of interval and timeout is seconds.
             if container.healthcheck:
@@ -340,6 +342,16 @@ class DockerDriver(driver.ContainerDriver):
     def get_host_default_base_size(self):
         return self.base_device_size
 
+    def _process_exposed_ports(self, neutron_api, container):
+        if not container.exposed_ports:
+            return
+
+        secgroup_name = self._get_secgorup_name(container.uuid)
+        secgroup_id = neutron_api.create_security_group({'security_group': {
+            "name": secgroup_name}})['security_group']['id']
+        neutron_api.expose_ports(secgroup_id, container.exposed_ports)
+        container.security_groups = [secgroup_id]
+
     def _process_networking_config(self, context, container,
                                    requested_networks, host_config,
                                    container_kwargs, docker):
@@ -352,7 +364,8 @@ class DockerDriver(driver.ContainerDriver):
         security_group_ids = utils.get_security_group_ids(
             context, container.security_groups)
         addresses, port = network_api.create_or_update_port(
-            container, docker_net_name, requested_network, security_group_ids)
+            container, docker_net_name, requested_network, security_group_ids,
+            set_binding_host=True)
         container.addresses = {requested_network['network']: addresses}
 
         ipv4_address = None
@@ -377,11 +390,14 @@ class DockerDriver(driver.ContainerDriver):
             self._get_or_create_docker_network(
                 context, network_api, rq_network['network'])
 
+    def _get_secgorup_name(self, container_uuid):
+        return consts.NAME_PREFIX + container_uuid
+
     def _get_binds(self, context, requested_volumes):
         binds = {}
         for volume in requested_volumes:
-            source, destination = self.volume_driver.bind_mount(context,
-                                                                volume)
+            volume_driver = self._get_volume_driver(volume)
+            source, destination = volume_driver.bind_mount(context, volume)
             binds[source] = {'bind': destination}
         return binds
 
@@ -407,15 +423,13 @@ class DockerDriver(driver.ContainerDriver):
         return addresses
 
     def delete(self, context, container, force):
-        teardown_network = True
-        if container.get_sandbox_id():
-            teardown_network = False
         with docker_utils.docker_client() as docker:
             try:
-                if teardown_network:
-                    network_api = zun_network.api(context=context,
-                                                  docker_api=docker)
-                    self._cleanup_network_for_container(container, network_api)
+                network_api = zun_network.api(context=context,
+                                              docker_api=docker)
+                self._cleanup_network_for_container(container, network_api)
+                self._cleanup_exposed_ports(network_api.neutron_api,
+                                            container)
                 if container.container_id:
                     docker.remove_container(container.container_id,
                                             force=force)
@@ -435,6 +449,15 @@ class DockerDriver(driver.ContainerDriver):
             network_api.disconnect_container_from_network(
                 container, docker_net, neutron_network_id=neutron_net)
 
+    def _cleanup_exposed_ports(self, neutron_api, container):
+        if not container.exposed_ports:
+            return
+
+        try:
+            neutron_api.delete_security_group(container.security_groups[0])
+        except n_exc.NeutronClientException:
+            LOG.exception("Failed to delete security group")
+
     def check_container_exist(self, container):
         with docker_utils.docker_client() as docker:
             docker_containers = [c['Id']
@@ -469,19 +492,24 @@ class DockerDriver(driver.ContainerDriver):
 
         return local_containers, non_existent_containers
 
-    def heal_with_rebuilding_container(self, context, container):
-        compute_api = zun_compute.API(context)
-        rebuild_status = [consts.CREATED, consts.RUNNING, consts.STOPPED]
+    def heal_with_rebuilding_container(self, context, container, manager):
+        if not container.container_id:
+            return
+
+        rebuild_status = utils.VALID_STATES['rebuild']
         try:
             if (container.auto_heal and
                     container.status in rebuild_status):
                 context.project_id = container.project_id
-                compute_api.container_rebuild(context, container)
+                objects.ContainerAction.action_start(
+                    context, container.uuid, container_actions.REBUILD,
+                    want_result=False)
+                manager.container_rebuild(context, container)
             else:
                 LOG.warning("Container %s was recorded in DB but "
                             "missing in docker", container.uuid)
                 container.status = consts.ERROR
-                msg = "No such container:%s in docker" % \
+                msg = "No such container: %s in docker" % \
                       (container.container_id)
                 container.status_reason = six.text_type(msg)
                 container.save(context)
@@ -503,7 +531,7 @@ class DockerDriver(driver.ContainerDriver):
                                             filters={'uuid': uuids})
         return containers
 
-    def update_containers_states(self, context, containers):
+    def update_containers_states(self, context, containers, manager):
         local_containers, non_existent_containers = self.list(context)
         if not local_containers:
             return
@@ -536,13 +564,13 @@ class DockerDriver(driver.ContainerDriver):
                 LOG.info('Host of container %s changed from %s to %s',
                          container.uuid, old_host, container.host)
         for container in non_existent_containers:
-            if container.host == cur_host:
+            if container.host == CONF.host:
                 if container.auto_remove:
                     container.status = consts.DELETED
                     container.save(context)
                 else:
-                    self.heal_with_rebuilding_container(context,
-                                                        container)
+                    self.heal_with_rebuilding_container(context, container,
+                                                        manager)
 
     def show(self, context, container):
         with docker_utils.docker_client() as docker:
@@ -957,53 +985,34 @@ class DockerDriver(driver.ContainerDriver):
             value = six.text_type(value)
         return value.encode('utf-8')
 
-    def create_sandbox(self, context, container, requested_networks,
-                       requested_volumes,
-                       image='kubernetes/pause'):
-        with docker_utils.docker_client() as docker:
-            network_api = zun_network.api(context=context, docker_api=docker)
-            self._provision_network(context, network_api, requested_networks)
-            binds = self._get_binds(context, requested_volumes)
-            host_config = {'binds': binds}
-            name = self.get_sandbox_name(container)
-            volumes = [b['bind'] for b in binds.values()]
-            kwargs = {
-                'name': name,
-                'hostname': name[:63],
-                'volumes': volumes,
-            }
-            self._process_networking_config(
-                context, container, requested_networks, host_config,
-                kwargs, docker)
-            kwargs['host_config'] = docker.create_host_config(**host_config)
-            sandbox = docker.create_container(image, **kwargs)
-            container.set_sandbox_id(sandbox['Id'])
-            addresses = self._setup_network_for_container(
-                context, container, requested_networks, network_api)
-            if not addresses:
-                raise exception.ZunException(_(
-                    "Unexpected missing of addresses"))
+    def _get_volume_driver(self, volume_mapping):
+        driver_name = volume_mapping.volume_provider
+        driver = self.volume_drivers.get(driver_name)
+        if not driver:
+            msg = _("The volume provider '%s' is not supported") % driver_name
+            raise exception.ZunException(msg)
 
-            container.addresses = addresses
-            container.save(context)
-
-            docker.start(sandbox['Id'])
-            return sandbox['Id']
+        return driver
 
     def attach_volume(self, context, volume_mapping):
-        self.volume_driver.attach(context, volume_mapping)
+        volume_driver = self._get_volume_driver(volume_mapping)
+        volume_driver.attach(context, volume_mapping)
 
     def detach_volume(self, context, volume_mapping):
-        self.volume_driver.detach(context, volume_mapping)
+        volume_driver = self._get_volume_driver(volume_mapping)
+        volume_driver.detach(context, volume_mapping)
 
     def delete_volume(self, context, volume_mapping):
-        self.volume_driver.delete(context, volume_mapping)
+        volume_driver = self._get_volume_driver(volume_mapping)
+        volume_driver.delete(context, volume_mapping)
 
-    def get_volume_status(self, context, volume_mapping):
-        return self.volume_driver.get_volume_status(context, volume_mapping)
+    def is_volume_available(self, context, volume_mapping):
+        volume_driver = self._get_volume_driver(volume_mapping)
+        return volume_driver.is_volume_available(context, volume_mapping)
 
-    def check_multiattach(self, context, volume_mapping):
-        return self.volume_driver.check_multiattach(context, volume_mapping)
+    def is_volume_deleted(self, context, volume_mapping):
+        volume_driver = self._get_volume_driver(volume_mapping)
+        return volume_driver.is_volume_deleted(context, volume_mapping)
 
     def _get_or_create_docker_network(self, context, network_api,
                                       neutron_net_id):
@@ -1020,25 +1029,6 @@ class DockerDriver(driver.ContainerDriver):
         # so it will not be duplicated across projects.
         return neutron_net_id
 
-    def delete_sandbox(self, context, container):
-        sandbox_id = container.get_sandbox_id()
-        with docker_utils.docker_client() as docker:
-            network_api = zun_network.api(context=context, docker_api=docker)
-            self._cleanup_network_for_container(container, network_api)
-            try:
-                docker.remove_container(sandbox_id, force=True)
-            except errors.APIError as api_error:
-                if is_not_found(api_error):
-                    return
-                raise
-
-    def stop_sandbox(self, context, sandbox_id):
-        with docker_utils.docker_client() as docker:
-            docker.stop(sandbox_id)
-
-    def get_sandbox_name(self, container):
-        return consts.SANDBOX_NAME_PREFIX + container.uuid
-
     def get_container_name(self, container):
         return consts.NAME_PREFIX + container.uuid
 
@@ -1067,20 +1057,33 @@ class DockerDriver(driver.ContainerDriver):
                     runtimes.append(key)
             else:
                 runtimes = ['runc']
-            return (total, running, paused, stopped, cpus,
-                    architecture, os_type, os, kernel_version, labels,
-                    runtimes)
+            docker_root_dir = info['DockerRootDir']
+            enable_cpu_pinning = CONF.compute.enable_cpu_pinning
+
+            return {'total_containers': total,
+                    'running_containers': running,
+                    'paused_containers': paused,
+                    'stopped_containers': stopped,
+                    'cpus': cpus,
+                    'architecture': architecture,
+                    'os_type': os_type,
+                    'os': os,
+                    'kernel_version': kernel_version,
+                    'labels': labels,
+                    'runtimes': runtimes,
+                    'docker_root_dir': docker_root_dir,
+                    'enable_cpu_pinning': enable_cpu_pinning}
 
     def get_total_disk_for_container(self):
         try:
-            (output, err) = utils.execute('df', '-B', '1G',
-                                          CONF.docker.docker_data_root,
-                                          run_as_root=True)
-        except exception.CommandError:
-            LOG.info('There was a problem while executing df -B 1G %s',
-                     CONF.docker.docker_data_root)
-            raise exception.CommandError(cmd='df')
-        total_disk = int(output.split('\n')[1].split()[1])
+            disk_usage = psutil.disk_usage(self.docker_root_dir)
+        except OSError as e:
+            if e.errno != errno.ENOENT:
+                raise
+            LOG.warning('Docker data root doesnot exist.')
+            # give another try with system root
+            disk_usage = psutil.disk_usage('/')
+        total_disk = disk_usage.total / 1024 ** 3
         return int(total_disk * (1 - CONF.compute.reserve_disk_for_image))
 
     def get_cpu_used(self):
@@ -1149,9 +1152,9 @@ class DockerDriver(driver.ContainerDriver):
                                           docker_api=docker)
             network = requested_network['network']
             if network in container.addresses:
-                raise exception.ZunException('Container %(container)s has'
-                                             ' alreay connected to the network'
-                                             '%(network)s.'
+                raise exception.ZunException('Container %(container)s has '
+                                             'already connected to the '
+                                             'network %(network)s.'
                                              % {'container': container.uuid,
                                                 'network': network})
             self._get_or_create_docker_network(context, network_api, network)
@@ -1169,13 +1172,136 @@ class DockerDriver(driver.ContainerDriver):
             container.addresses = addresses
             container.save(context)
 
-    def create_network(self, context, network):
+    def create_network(self, context, neutron_net_id):
         with docker_utils.docker_client() as docker:
             network_api = zun_network.api(context,
                                           docker_api=docker)
             docker_net_name = self._get_docker_network_name(
-                context, network.neutron_net_id)
-            docker_network = network_api.create_network(
-                neutron_net_id=network.neutron_net_id,
+                context, neutron_net_id)
+            return network_api.create_network(
+                neutron_net_id=neutron_net_id,
                 name=docker_net_name)
-            return docker_network
+
+    def delete_network(self, context, network):
+        with docker_utils.docker_client() as docker:
+            network_api = zun_network.api(context,
+                                          docker_api=docker)
+            network_api.remove_network(network)
+
+    def create_capsule(self, context, capsule, image, requested_networks,
+                       requested_volumes):
+        capsule = self.create(context, capsule, image, requested_networks,
+                              requested_volumes)
+        self.start(context, capsule)
+        for container in capsule.containers:
+            self._create_container_in_capsule(context, capsule, container,
+                                              requested_networks,
+                                              requested_volumes)
+        return capsule
+
+    def _create_container_in_capsule(self, context, capsule, container,
+                                     requested_networks, requested_volumes):
+        # pull image
+        image_driver_name = container.image_driver
+        repo, tag = utils.parse_image_name(container.image, image_driver_name)
+        image_pull_policy = utils.get_image_pull_policy(
+            container.image_pull_policy, tag)
+        image, image_loaded = self.pull_image(
+            context, repo, tag, image_pull_policy, image_driver_name)
+        image['repo'], image['tag'] = repo, tag
+        if not image_loaded:
+            self.load_image(image['path'])
+        if image_driver_name == 'glance':
+            self.read_tar_image(image)
+        if image['tag'] != tag:
+            LOG.warning("The input tag is different from the tag in tar")
+
+        # create container
+        with docker_utils.docker_client() as docker:
+            name = container.name
+            LOG.debug('Creating container with image %(image)s name %(name)s',
+                      {'image': image['image'], 'name': name})
+            volmaps = requested_volumes.get(container.uuid, [])
+            binds = self._get_binds(context, volmaps)
+            kwargs = {
+                'name': self.get_container_name(container),
+                'command': container.command,
+                'environment': container.environment,
+                'working_dir': container.workdir,
+                'labels': container.labels,
+                'tty': container.interactive,
+                'stdin_open': container.interactive,
+            }
+
+            host_config = {}
+            host_config['privileged'] = container.privileged
+            host_config['binds'] = binds
+            kwargs['volumes'] = [b['bind'] for b in binds.values()]
+            host_config['network_mode'] = 'container:%s' % capsule.container_id
+            # TODO(hongbin): Uncomment this after docker-py add support for
+            # container mode for pid namespace.
+            # host_config['pid_mode'] = 'container:%s' % capsule.container_id
+            host_config['ipc_mode'] = 'container:%s' % capsule.container_id
+            if container.auto_remove:
+                host_config['auto_remove'] = container.auto_remove
+            if container.memory is not None:
+                host_config['mem_limit'] = str(container.memory) + 'M'
+            if container.cpu is not None:
+                host_config['cpu_quota'] = int(100000 * container.cpu)
+                host_config['cpu_period'] = 100000
+            if container.restart_policy:
+                count = int(container.restart_policy['MaximumRetryCount'])
+                name = container.restart_policy['Name']
+                host_config['restart_policy'] = {'Name': name,
+                                                 'MaximumRetryCount': count}
+
+            if container.disk:
+                disk_size = str(container.disk) + 'G'
+                host_config['storage_opt'] = {'size': disk_size}
+            # The time unit in docker of heath checking is us, and the unit
+            # of interval and timeout is seconds.
+            if container.healthcheck:
+                healthcheck = {}
+                healthcheck['test'] = container.healthcheck.get('test', '')
+                interval = container.healthcheck.get('interval', 0)
+                healthcheck['interval'] = interval * 10 ** 9
+                healthcheck['retries'] = int(container.healthcheck.
+                                             get('retries', 0))
+                timeout = container.healthcheck.get('timeout', 0)
+                healthcheck['timeout'] = timeout * 10 ** 9
+                kwargs['healthcheck'] = healthcheck
+
+            kwargs['host_config'] = docker.create_host_config(**host_config)
+            if image['tag']:
+                image_repo = image['repo'] + ":" + image['tag']
+            else:
+                image_repo = image['repo']
+            response = docker.create_container(image_repo, **kwargs)
+            container.container_id = response['Id']
+            docker.start(container.container_id)
+
+            response = docker.inspect_container(container.container_id)
+            self._populate_container(container, response)
+            container.save(context)
+
+    def delete_capsule(self, context, capsule, force):
+        for container in capsule.containers:
+            self._delete_container_in_capsule(context, capsule, container,
+                                              force)
+        self.delete(context, capsule, force)
+
+    def _delete_container_in_capsule(self, context, capsule, container, force):
+        if not container.container_id:
+            return
+
+        with docker_utils.docker_client() as docker:
+            try:
+                docker.stop(container.container_id)
+                docker.remove_container(container.container_id,
+                                        force=force)
+            except errors.APIError as api_error:
+                if is_not_found(api_error):
+                    return
+                if is_not_connected(api_error):
+                    return
+                raise
