@@ -37,14 +37,15 @@ def _get_dbdriver_instance():
 
 
 @profiler.trace("db")
-def list_containers(context, filters=None, limit=None, marker=None,
-                    sort_key=None, sort_dir=None):
+def list_containers(context, container_type, filters=None, limit=None,
+                    marker=None, sort_key=None, sort_dir=None):
     """List matching containers.
 
     Return a list of the specified columns for all containers that match
     the specified filters.
 
     :param context: The security context
+    :param container_type: The container type
     :param filters: Filters to apply. Defaults to None.
     :param limit: Maximum number of containers to return.
     :param marker: the last item of the previous page; we return the next
@@ -55,7 +56,7 @@ def list_containers(context, filters=None, limit=None, marker=None,
     :returns: A list of tuples of the specified columns.
     """
     return _get_dbdriver_instance().list_containers(
-        context, filters, limit, marker, sort_key, sort_dir)
+        context, container_type, filters, limit, marker, sort_key, sort_dir)
 
 
 @profiler.trace("db")
@@ -81,51 +82,56 @@ def create_container(context, values):
 
 
 @profiler.trace("db")
-def get_container_by_uuid(context, container_uuid):
+def get_container_by_uuid(context, container_type, container_uuid):
     """Return a container.
 
     :param context: The security context
+    :param container_type: The container type
     :param container_uuid: The uuid of a container.
     :returns: A container.
     """
     return _get_dbdriver_instance().get_container_by_uuid(
-        context, container_uuid)
+        context, container_type, container_uuid)
 
 
 @profiler.trace("db")
-def get_container_by_name(context, container_name):
+def get_container_by_name(context, container_type, container_name):
     """Return a container.
 
     :param context: The security context
+    :param container_type: The container type
     :param container_name: The name of a container.
     :returns: A container.
     """
     return _get_dbdriver_instance().get_container_by_name(
-        context, container_name)
+        context, container_type, container_name)
 
 
 @profiler.trace("db")
-def destroy_container(context, container_id):
+def destroy_container(context, container_type, container_id):
     """Destroy a container and all associated interfaces.
 
     :param context: Request context
+    :param container_type: The container type
     :param container_id: The id or uuid of a container.
     """
-    return _get_dbdriver_instance().destroy_container(context, container_id)
+    return _get_dbdriver_instance().destroy_container(
+        context, container_type, container_id)
 
 
 @profiler.trace("db")
-def update_container(context, container_id, values):
+def update_container(context, container_type, container_id, values):
     """Update properties of a container.
 
     :param context: Request context
+    :param container_type: The container type
     :param container_id: The id or uuid of a container.
     :param values: The properties to be updated
     :returns: A container.
     :raises: ContainerNotFound
     """
     return _get_dbdriver_instance().update_container(
-        context, container_id, values)
+        context, container_type, container_id, values)
 
 
 @profiler.trace("db")
@@ -150,6 +156,20 @@ def list_volume_mappings(context, filters=None, limit=None, marker=None,
         context, filters, limit, marker, sort_key, sort_dir)
 
 
+@profiler.trace("db")
+def count_volume_mappings(context, **filters):
+    """Count matching volume mappings.
+
+    Return the count of all volume mappings that match
+    the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply.
+    :returns: The count of volume mapping.
+    """
+    return _get_dbdriver_instance().count_volume_mappings(context, **filters)
+
+
 @profiler.trace("db")
 def create_volume_mapping(context, values):
     """Create a volume mapping.
@@ -198,6 +218,54 @@ def update_volume_mapping(context, vm_id, values):
         context, vm_id, values)
 
 
+@profiler.trace("db")
+def create_volume(context, values):
+    """Create a volume.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the volume.
+    :returns: A volume.
+    """
+    return _get_dbdriver_instance().create_volume(context, values)
+
+
+@profiler.trace("db")
+def get_volume_by_id(context, vol_id):
+    """Return a volume
+
+    :param context: The security context
+    :param vol_id: The id of a volume.
+    :returns: A volume.
+    """
+    return _get_dbdriver_instance().get_volume_by_id(
+        context, vol_id)
+
+
+@profiler.trace("db")
+def destroy_volume(context, vol_id):
+    """Destroy a volume.
+
+    :param context: Request context
+    :param vm_id: The id or uuid of a volume.
+    """
+    return _get_dbdriver_instance().destroy_volume(context, vol_id)
+
+
+@profiler.trace("db")
+def update_volume(context, vol_id, values):
+    """Update properties of a volume.
+
+    :param context: Request context
+    :param vm_id: The id or uuid of a volume.
+    :param values: The properties to be updated
+    :returns: A volume.
+    :raises: VolumeNotFound
+    """
+    return _get_dbdriver_instance().update_volume(
+        context, vol_id, values)
+
+
 @profiler.trace("db")
 def destroy_zun_service(host, binary):
     """Destroys a zun_service record.
@@ -733,97 +801,6 @@ def update_compute_node(context, node_uuid, values):
         context, node_uuid, values)
 
 
-@profiler.trace("db")
-def list_capsules(context, filters=None, limit=None, marker=None,
-                  sort_key=None, sort_dir=None):
-    """List matching capsules.
-
-    Return a list of the specified columns for all capsules that match
-    the specified filters.
-
-    :param context: The security context
-    :param filters: Filters to apply. Defaults to None.
-    :param limit: Maximum number of capsules to return.
-    :param marker: the last item of the previous page; we return the next
-                   result set.
-    :param sort_key: Attribute by which results should be sorted.
-    :param sort_dir: Direction in which results should be sorted.
-                     (asc, desc)
-    :returns: A list of tuples of the specified columns.
-    """
-    return _get_dbdriver_instance().list_capsules(
-        context, filters, limit, marker, sort_key, sort_dir)
-
-
-@profiler.trace("db")
-def create_capsule(context, values):
-    """Create a new capsule.
-
-    :param context: The security context
-    :param values: A dict containing several items used to identify
-                   and track the container, and several dicts which are
-                   passed into the Drivers when managing this container.
-                   For example:
-                   ::
-
-                    {
-                     'uuid': uuidutils.generate_uuid(),
-                     'restart_policy': 'always',
-                     'project_id': '***'
-                    }
-    :returns: A capsule.
-    """
-    return _get_dbdriver_instance().create_capsule(context, values)
-
-
-@profiler.trace("db")
-def get_capsule_by_uuid(context, capsule_uuid):
-    """Return a capsule.
-
-    :param context: The security context
-    :param capsule_uuid: The uuid of a capsule.
-    :returns: A capsule.
-    """
-    return _get_dbdriver_instance().get_capsule_by_uuid(
-        context, capsule_uuid)
-
-
-@profiler.trace("db")
-def get_capsule_by_meta_name(context, capsule_name):
-    """Return a capsule.
-
-    :param context: The security context
-    :param capsule_name: The meta_name of a capsule.
-    :returns: A capsule.
-    """
-    return _get_dbdriver_instance().get_capsule_by_meta_name(
-        context, capsule_name)
-
-
-@profiler.trace("db")
-def destroy_capsule(context, capsule_id):
-    """Destroy a capsule and all associated interfaces.
-
-    :param context: Request context
-    :param capsule_id: The id or uuid of a capsule.
-    """
-    return _get_dbdriver_instance().destroy_capsule(context, capsule_id)
-
-
-@profiler.trace("db")
-def update_capsule(context, capsule_id, values):
-    """Update properties of a capsule.
-
-    :param context: Request context
-    :param capsule_id: The id or uuid of a capsule.
-    :param values: The properties to be updated
-    :returns: A capsule.
-    :raises: CapsuleNotFound
-    """
-    return _get_dbdriver_instance().update_capsule(
-        context, capsule_id, values)
-
-
 @profiler.trace("db")
 def get_pci_device_by_addr(node_id, dev_addr):
     """Get PCI device by address."""
@@ -1019,6 +996,28 @@ def get_network_by_uuid(context, network_uuid):
     return _get_dbdriver_instance().get_network_by_uuid(context, network_uuid)
 
 
+@profiler.trace("db")
+def list_networks(context, filters=None, limit=None, marker=None,
+                  sort_key=None, sort_dir=None):
+    """List matching networks.
+
+    Return a list of the specified columns for all networks that match
+    the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply. Defaults to None.
+    :param limit: Maximum number of networks to return.
+    :param marker: the last item of the previous page; we return the next
+                   result set.
+    :param sort_key: Attribute by which results should be sorted.
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
+    :returns: A list of tuples of the specified columns.
+    """
+    return _get_dbdriver_instance().list_networks(
+        context, filters, limit, marker, sort_key, sort_dir)
+
+
 @profiler.trace("db")
 def update_network(context, uuid, values):
     """Update properties of a network.
@@ -1032,6 +1031,16 @@ def update_network(context, uuid, values):
         context, uuid, values)
 
 
+@profiler.trace("db")
+def destroy_network(context, network_uuid):
+    """Destroy a network.
+
+    :param context: Request context
+    :param network_uuid: The uuid of a network.
+    """
+    return _get_dbdriver_instance().destroy_network(context, network_uuid)
+
+
 @profiler.trace("db")
 def create_exec_instance(context, values):
     """Create a new exec instance.
@@ -1067,5 +1076,101 @@ def list_exec_instances(context, filters=None, limit=None, marker=None,
 
 
 @profiler.trace('db')
-def count_usage(context, project_id, flag):
-    return _get_dbdriver_instance().count_usage(context, project_id, flag)
+def count_usage(context, container_type, project_id, flag):
+    return _get_dbdriver_instance().count_usage(context, container_type,
+                                                project_id, flag)
+
+
+@profiler.trace("db")
+def create_registry(context, values):
+    """Create a new registry.
+
+    :param context: The security context
+    :param values: A dict containing several items used to identify
+                   and track the registry, and several dicts which are
+                   passed
+                   into the Drivers when managing this registry.
+    :returns: A registry.
+    """
+    return _get_dbdriver_instance().create_registry(context, values)
+
+
+@profiler.trace("db")
+def get_registry_by_id(context, registry_id):
+    """Return a registry.
+
+    :param context: The security context
+    :param registry_id: The id of a registry.
+    :returns: A registry.
+    """
+    return _get_dbdriver_instance().get_registry_by_id(
+        context, registry_id)
+
+
+@profiler.trace("db")
+def get_registry_by_uuid(context, registry_uuid):
+    """Return a registry.
+
+    :param context: The security context
+    :param registry_uuid: The uuid of a registry.
+    :returns: A registry.
+    """
+    return _get_dbdriver_instance().get_registry_by_uuid(
+        context, registry_uuid)
+
+
+@profiler.trace("db")
+def get_registry_by_name(context, registry_name):
+    """Return a registry.
+
+    :param context: The security context
+    :param registry_name: The name of a registry.
+    :returns: A registry.
+    """
+    return _get_dbdriver_instance().get_registry_by_name(
+        context, registry_name)
+
+
+@profiler.trace("db")
+def list_registries(context, filters=None, limit=None, marker=None,
+                    sort_key=None, sort_dir=None):
+    """List matching registries.
+
+    Return a list of the specified columns for all registries that match
+    the specified filters.
+
+    :param context: The security context
+    :param filters: Filters to apply. Defaults to None.
+    :param limit: Maximum number of registries to return.
+    :param marker: the last item of the previous page; we return the next
+                   result set.
+    :param sort_key: Attribute by which results should be sorted.
+    :param sort_dir: Direction in which results should be sorted.
+                     (asc, desc)
+    :returns: A list of tuples of the specified columns.
+    """
+    return _get_dbdriver_instance().list_registries(
+        context, filters, limit, marker, sort_key, sort_dir)
+
+
+@profiler.trace("db")
+def update_registry(context, uuid, values):
+    """Update properties of a registry.
+
+    :param context: Request context
+    :param uuid: The id or uuid of a registry.
+    :param values: The properties to be updated
+    :returns: A registry.
+    """
+    return _get_dbdriver_instance().update_registry(
+        context, uuid, values)
+
+
+@profiler.trace("db")
+def destroy_registry(context, registry_uuid):
+    """Destroy a registry.
+
+    :param context: Request context
+    :param registry_uuid: The uuid of a registry.
+    """
+    return _get_dbdriver_instance().destroy_registry(context, registry_uuid)
