@@ -10,7 +10,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from zun.objects import capsule
 from zun.objects import compute_node
 from zun.objects import container
 from zun.objects import container_action
@@ -23,13 +22,16 @@ from zun.objects import pci_device
 from zun.objects import pci_device_pool
 from zun.objects import quota
 from zun.objects import quota_class
+from zun.objects import registry
 from zun.objects import resource_class
 from zun.objects import resource_provider
+from zun.objects import volume
 from zun.objects import volume_mapping
 from zun.objects import zun_service
 
 
 Container = container.Container
+Volume = volume.Volume
 VolumeMapping = volume_mapping.VolumeMapping
 ZunService = zun_service.ZunService
 Image = image.Image
@@ -39,7 +41,9 @@ NUMATopology = numa.NUMATopology
 ResourceProvider = resource_provider.ResourceProvider
 ResourceClass = resource_class.ResourceClass
 ComputeNode = compute_node.ComputeNode
-Capsule = capsule.Capsule
+Capsule = container.Capsule
+CapsuleContainer = container.CapsuleContainer
+CapsuleInitContainer = container.CapsuleInitContainer
 PciDevice = pci_device.PciDevice
 PciDevicePool = pci_device_pool.PciDevicePool
 Quota = quota.Quota
@@ -49,6 +53,7 @@ ContainerPCIRequests = container_pci_requests.ContainerPCIRequests
 ContainerAction = container_action.ContainerAction
 ContainerActionEvent = container_action.ContainerActionEvent
 ExecInstance = exec_instance.ExecInstance
+Registry = registry.Registry
 
 __all__ = (
     'Container',
@@ -62,6 +67,8 @@ __all__ = (
     'NUMATopology',
     'ComputeNode',
     'Capsule',
+    'CapsuleContainer',
+    'CapsuleInitContainer',
     'PciDevice',
     'PciDevicePool',
     'Quota',
@@ -71,4 +78,5 @@ __all__ = (
     'ContainerAction',
     'ContainerActionEvent',
     'ExecInstance',
+    'Registry',
 )
