@@ -59,6 +59,27 @@ class TestAPI(base.TestCase):
         self.assertTrue(mock_image_search.called)
         self.assertTrue(mock_container_create.called)
 
+    @mock.patch('zun.compute.api.API._record_action_start')
+    @mock.patch('zun.compute.rpcapi.API.container_create')
+    @mock.patch('zun.compute.rpcapi.API.image_search')
+    @mock.patch('zun.compute.api.API._schedule_container')
+    def test_container_create_with_private_registry_image(
+            self, mock_schedule_container, mock_image_search,
+            mock_container_create, mock_record_action_start):
+        container = self.container
+        container.image = 'myregistry.io/test-image'
+        container.image_driver = 'docker'
+        mock_schedule_container.return_value = {'host': u'Centos',
+                                                'nodename': None,
+                                                'limits': {}}
+        mock_image_search.side_effect = exception.OperationNotSupported
+
+        self.compute_api.container_create(self.context, container,
+                                          None, None, None, False)
+        self.assertTrue(mock_schedule_container.called)
+        self.assertTrue(mock_image_search.called)
+        self.assertTrue(mock_container_create.called)
+
     @mock.patch('zun.compute.api.API._schedule_container')
     @mock.patch.object(objects.Container, 'save')
     def test_schedule_container_exception(self, mock_save,
@@ -73,12 +94,15 @@ class TestAPI(base.TestCase):
         self.assertTrue(mock_save.called)
         self.assertEqual(consts.ERROR, container.status)
 
+    @mock.patch('zun.compute.rpcapi.API._cast')
+    @mock.patch.object(objects.ContainerAction, 'action_start')
     @mock.patch('zun.compute.rpcapi.API.image_search')
     @mock.patch('zun.compute.api.API._schedule_container')
     @mock.patch.object(objects.Container, 'save')
     def test_searching_image_exception(self, mock_save,
                                        mock_schedule_container,
-                                       mock_image_search):
+                                       mock_image_search,
+                                       mock_start, mock_cast):
         container = self.container
         container.status = consts.CREATING
         mock_schedule_container.return_value = {'host': u'Centos',
@@ -86,22 +110,12 @@ class TestAPI(base.TestCase):
                                                 'limits': {}}
         mock_image_search.side_effect = exception.ZunException
 
-        self.assertRaises(exception.ZunException,
-                          self.compute_api.container_create,
-                          self.context, container,
-                          None, None, None, False)
+        self.compute_api.container_create(
+            self.context, container, None, None, None, False)
         self.assertTrue(mock_schedule_container.called)
         self.assertTrue(mock_image_search.called)
-        self.assertTrue(mock_save.called)
-        self.assertEqual(consts.ERROR, container.status)
-
-    @mock.patch('zun.compute.rpcapi.API._call')
-    def test_capsule_delete(self, mock_call):
-        capsule = self.container
-        self.compute_api.capsule_delete(
-            self.context, capsule)
-        mock_call.assert_called_once_with(
-            capsule.host, "capsule_delete", capsule=capsule)
+        self.assertFalse(mock_save.called)
+        self.assertEqual(consts.CREATING, container.status)
 
     @mock.patch('zun.compute.rpcapi.API._cast')
     @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
@@ -369,10 +383,10 @@ class TestAPI(base.TestCase):
         mock_srv_list.return_value = [srv]
         mock_srv_up.return_value = True
         self.compute_api.container_get_archive(
-            self.context, container, "/root")
+            self.context, container, "/root", True)
         mock_call.assert_called_once_with(
             container.host, "container_get_archive",
-            container=container, path="/root")
+            container=container, path="/root", encode_data=True)
 
     @mock.patch('zun.compute.rpcapi.API._cast')
     @mock.patch.object(objects.ContainerAction, 'action_start')
@@ -412,10 +426,10 @@ class TestAPI(base.TestCase):
         mock_srv_list.return_value = [srv]
         mock_srv_up.return_value = True
         self.compute_api.container_put_archive(
-            self.context, container, "/root", {})
+            self.context, container, "/root", {}, True)
         mock_call.assert_called_once_with(
             container.host, "container_put_archive",
-            container=container, path="/root", data={})
+            container=container, path="/root", data={}, decode_data=True)
 
     @mock.patch('zun.compute.rpcapi.API._call')
     @mock.patch('zun.api.servicegroup.ServiceGroup.service_is_up')
@@ -457,9 +471,9 @@ class TestAPI(base.TestCase):
     @mock.patch('zun.compute.rpcapi.API._call')
     def test_image_search(self, mock_call):
         self.compute_api.image_search(
-            self.context, "ubuntu", "glance", True)
+            self.context, "ubuntu", "glance", True, None)
         mock_call.assert_called_once_with(
-            None, "image_search", image="ubuntu",
+            None, "image_search", image="ubuntu", registry=None,
             image_driver_name="glance", exact_match=True)
 
     @mock.patch('zun.compute.rpcapi.API._cast')
